@@ -22,6 +22,8 @@
 #include "common_defs.h"
 #include "prefs.h"
 
+extern int make_soft_link(PROJECT* project, char* link_path, char* rel_file_path);
+
 // get the pathname (relative to client home dir) of a project file
 //
 extern void get_pathname(FILE_INFO* fip, char* path, int len);
@@ -73,7 +75,6 @@ extern void send_log_after(const char* filename, double t, MIOFILE& mf);
 #define REMOTEHOST_FILE_NAME        "remote_hosts.cfg"
 #define ACCT_MGR_REQUEST_FILENAME   "acct_mgr_request.xml"
 #define ACCT_MGR_REPLY_FILENAME     "acct_mgr_reply.xml"
-#define GUI_RPC_PASSWD_FILE         "gui_rpc_auth.cfg"
 #define PROJECT_INIT_FILENAME       "project_init.xml"
 #define ACCT_MGR_URL_FILENAME       "acct_mgr_url.xml"
 #define ACCT_MGR_LOGIN_FILENAME     "acct_mgr_login.xml"
@@ -91,5 +92,5 @@ extern void send_log_after(const char* filename, double t, MIOFILE& mf);
 #define CA_BUNDLE_FILENAME          "ca-bundle.crt"
 #define CLIENT_AUTH_FILENAME        "client_auth.xml"
 #define CERTIFICATE_DIRECTORY       "certificates"
-
+#define TASK_STATE_FILENAME         "boinc_task_state.xml"
 #endif
