@@ -15,8 +15,6 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-#include "cpp.h"
-
 #ifdef _WIN32
 #include "boinc_win.h"
 #else
@@ -32,54 +30,53 @@
 #endif
 #endif
 
+#include "cpp.h"
 #include "error_numbers.h"
 #include "filesys.h"
 #include "client_msgs.h"
 #include "log_flags.h"
 #include "str_util.h"
+#include "str_replace.h"
 #include "util.h"
 
 #include "network.h"
 #include "file_names.h"
 #include "client_msgs.h"
 #include "base64.h"
-#include "http_curl.h"
 #include "client_state.h"
+#include "http_curl.h"
 
 using std::min;
 using std::vector;
 
 static CURLM* g_curlMulti = NULL;
-
 static char g_user_agent_string[256] = {""};
 static const char g_content_type[] = {"Content-Type: application/x-www-form-urlencoded"};
 
 
-// Breaks a HTTP URL down into its server, port and file components
+// Breaks a URL down into its protocol, server, port and file components
 // format of url:
 // [http[s]://]host.dom.dom[:port][/dir/file]
+// [socks]://]host.dom.dom[:port][/dir/file]
 //
-void parse_url(const char* url, char* host, int &port, char* file) {
+void parse_url(const char* url, int &protocol, char* host, int &port, char* file) {
     char* p;
     char buf[256];
-    bool bSSL = false;
 
-    //const short csiLen = bSSL ? 8 : 7; // get right size of http/s string comparator
-
-    // strip off http:// if present
+    // strip off the protocol if present
     //
-    //if (strncmp(url, (bSSL ? "https://" : "http://"), csiLen) == 0) {
     if (strncmp(url, "http://", 7) == 0) {
         safe_strcpy(buf, url+7);
-    } else { 
-        // wait, may be https://
-        if (strncmp(url, "https://", 8) == 0) {
-            safe_strcpy(buf, url+8);
-            bSSL = true; // retain the fact that this was a secure http url
-        }
-        else { // no http:// or https:// prepended on url
-            safe_strcpy(buf, url);
-        }
+        protocol = URL_PROTOCOL_HTTP;
+    } else if (strncmp(url, "https://", 8) == 0) {
+        safe_strcpy(buf, url+8);
+        protocol = URL_PROTOCOL_HTTPS;
+    } else if (strncmp(url, "socks://", 8) == 0) {
+        safe_strcpy(buf, url+8);
+        protocol = URL_PROTOCOL_SOCKS;
+    } else {
+        safe_strcpy(buf, url);
+        protocol = URL_PROTOCOL_UNKNOWN;
     }
 
     // parse and strip off file part if present
@@ -99,10 +96,10 @@ void parse_url(const char* url, char* host, int &port, char* file) {
         port = atol(p+1);
         *p = 0;
     } else {
-        // CMC note:  if they didn't pass in a port #, 
+        // CMC note:  if they didn't pass in a port #,
         //    but the url starts with https://, assume they
         //    want a secure port (HTTPS, port 443)
-        port = (bSSL ? 443 : 80);  
+        port = (protocol == URL_PROTOCOL_HTTPS) ? 443 : 80;
     }
 
     // what remains is the host
@@ -110,17 +107,18 @@ void parse_url(const char* url, char* host, int &port, char* file) {
     strcpy(host, buf);
 }
 
-void get_user_agent_string() {
+char* get_user_agent_string() {
     sprintf(g_user_agent_string, "BOINC client (%s %d.%d.%d)",
         gstate.get_primary_platform(),
         BOINC_MAJOR_VERSION, BOINC_MINOR_VERSION, BOINC_RELEASE
     );
+    return (char*)&g_user_agent_string;
 }
 
 void HTTP_OP::init() {
     reset();
     start_time = gstate.now;
-	start_bytes_xferred = 0;
+    start_bytes_xferred = 0;
 }
 
 void HTTP_OP::reset() {
@@ -181,7 +179,7 @@ int HTTP_OP::init_get(
     // usually have an outfile on a get
     if (off != 0) {
         bytes_xferred = off;
-		start_bytes_xferred = off;
+        start_bytes_xferred = off;
     }
     http_op_type = HTTP_OP_GET;
     http_op_state = HTTP_STATE_CONNECTING;
@@ -253,37 +251,38 @@ int HTTP_OP::init_post2(
 // is URL in proxy exception list?
 //
 bool HTTP_OP::no_proxy_for_url(const char* url) {
-	if (log_flags.proxy_debug) {
+    if (log_flags.proxy_debug) {
         msg_printf(0, MSG_INFO, "[proxy_debug] HTTP_OP::no_proxy_for_url(): %s", url);
     }
-	char hosturl[256];
-	char hostnoproxy[256];	
-	char file[256];
-	char noproxy[256];
-	int port;
+    char hosturl[256];
+    char file[256];
+    char hostnoproxy[256];
+    char noproxy[256];
+    int protocol;
+    int port;
 
-	// extract the host from the url
-	parse_url(url,hosturl,port,file);
+    // extract the host from the url
+    parse_url(url, protocol, hosturl, port, file);
 
-	// tokenize the noproxy-entry and check for identical hosts
+    // tokenize the noproxy-entry and check for identical hosts
     //
-	strcpy(noproxy,pi.noproxy_hosts);
-	char* token = strtok(noproxy,",");
-	while(token!= NULL) {
-		// extract the host from the no_proxy url
-		parse_url(token,hostnoproxy,port,file);
-		if (!strcmp(hostnoproxy,hosturl)) {
-			if (log_flags.proxy_debug) {
-				msg_printf(0, MSG_INFO, "[proxy_debug] disabling proxy for %s",url);
-			}
-			return true;
-		}
-		token = strtok(NULL,",");
-	}
-	if (log_flags.proxy_debug) {
-		msg_printf(0, MSG_INFO, "[proxy_debug] returning false");
-	}
-	return false;
+    strcpy(noproxy, gstate.proxy_info.noproxy_hosts);
+    char* token = strtok(noproxy, ",");
+    while(token!= NULL) {
+        // extract the host from the no_proxy url
+        parse_url(token, protocol, hostnoproxy, port, file);
+        if (hostnoproxy == hosturl) {
+            if (log_flags.proxy_debug) {
+                msg_printf(0, MSG_INFO, "[proxy_debug] disabling proxy for %s", url);
+            }
+            return true;
+        }
+        token = strtok(NULL, ",");
+    }
+    if (log_flags.proxy_debug) {
+        msg_printf(0, MSG_INFO, "[proxy_debug] returning false");
+    }
+    return false;
 }
 
 // the following will do an HTTP GET or POST using libcurl
@@ -316,12 +315,14 @@ int HTTP_OP::libcurl_exec(
 
     curlEasy = curl_easy_init(); // get a curl_easy handle to use
     if (!curlEasy) {
-        msg_printf(0, MSG_INTERNAL_ERROR, "Couldn't create curlEasy handle");
+        if (log_flags.http_debug) {
+            msg_printf(0, MSG_INFO, "Couldn't create curlEasy handle");
+        }
         return ERR_HTTP_ERROR; // returns 0 (CURLM_OK) on successful handle creation
     }
 
-	// the following seems to be a no-op
-	//curlErr = curl_easy_setopt(curlEasy, CURLOPT_ERRORBUFFER, error_msg);
+    // the following seems to be a no-op
+    //curlErr = curl_easy_setopt(curlEasy, CURLOPT_ERRORBUFFER, error_msg);
 
     curlErr = curl_easy_setopt(curlEasy, CURLOPT_URL, m_url);
 
@@ -344,7 +345,7 @@ int HTTP_OP::libcurl_exec(
     // The default, since 7.10, is 2.
     // The checking this option controls is of the identity that
     // the server claims. The server could be lying.
-    // To control lying, see CURLOPT_SSL_VERIFYPEER. 
+    // To control lying, see CURLOPT_SSL_VERIFYPEER.
     //
     curlErr = curl_easy_setopt(curlEasy, CURLOPT_SSL_VERIFYHOST, 2L);
 
@@ -369,12 +370,12 @@ int HTTP_OP::libcurl_exec(
 
             strncat(
                 m_curl_ca_bundle_location,
-                szPath, 
+                szPath,
                 sizeof(m_curl_ca_bundle_location)-strlen(m_curl_ca_bundle_location)
             );
             strncat(
                 m_curl_ca_bundle_location,
-                CA_BUNDLE_FILENAME, 
+                CA_BUNDLE_FILENAME,
                 sizeof(m_curl_ca_bundle_location)-strlen(m_curl_ca_bundle_location)
             );
 
@@ -427,13 +428,13 @@ int HTTP_OP::libcurl_exec(
     curlErr = curl_easy_setopt(curlEasy, CURLOPT_LOW_SPEED_LIMIT, 10L);
     curlErr = curl_easy_setopt(curlEasy, CURLOPT_LOW_SPEED_TIME, 300L);
     curlErr = curl_easy_setopt(curlEasy, CURLOPT_CONNECTTIMEOUT, 120L);
-    
+
     // force curl to use HTTP/1.0 if config specifies it
-	// (curl uses 1.1 by default)
-	//
-	if (config.http_1_0 || (config.force_auth == "ntlm")) {
+    // (curl uses 1.1 by default)
+    //
+    if (config.http_1_0 || (config.force_auth == "ntlm")) {
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
-	}
+    }
     curlErr = curl_easy_setopt(curlEasy, CURLOPT_MAXREDIRS, 50L);
     curlErr = curl_easy_setopt(curlEasy, CURLOPT_AUTOREFERER, 1L);
     curlErr = curl_easy_setopt(curlEasy, CURLOPT_FOLLOWLOCATION, 1L);
@@ -447,19 +448,19 @@ int HTTP_OP::libcurl_exec(
     if (!out || !ends_with(std::string(out), std::string(".gz"))) {
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_ENCODING, "");
     }
-	
-	// setup any proxy they may need
+
+    // setup any proxy they may need
     //
-    setupProxyCurl(no_proxy_for_url(url));
+    setup_proxy_session(no_proxy_for_url(url));
 
 
     // set the content type in the header
     //
     pcurlList = curl_slist_append(pcurlList, g_content_type);
 
-	// set the file offset for resumable downloads
+    // set the file offset for resumable downloads
     //
-	if (!bPost && offset>0.0f) {
+    if (!bPost && offset>0.0f) {
         file_offset = offset;
         sprintf(strTmp, "Range: bytes=%.0f-", offset);
         pcurlList = curl_slist_append(pcurlList, strTmp);
@@ -467,14 +468,14 @@ int HTTP_OP::libcurl_exec(
 
     // set up an output file for the reply
     //
-    if (strlen(outfile)) {  
-		if (file_offset>0.0) {
+    if (strlen(outfile)) {
+        if (file_offset>0.0) {
             fileOut = boinc_fopen(outfile, "ab+");
         } else {
             fileOut = boinc_fopen(outfile, "wb+");
         }
         if (!fileOut) {
-            msg_printf(NULL, MSG_INTERNAL_ERROR, 
+            msg_printf(NULL, MSG_INTERNAL_ERROR,
                 "Can't create HTTP response output file %s", outfile
             );
             http_op_retval = ERR_FOPEN;
@@ -502,7 +503,7 @@ int HTTP_OP::libcurl_exec(
                 http_op_state = HTTP_STATE_DONE;
                 return ERR_FOPEN;
             }
-        }        
+        }
 
         if (pcurlList) { // send custom headers if required
             curlErr = curl_easy_setopt(curlEasy, CURLOPT_HTTPHEADER, pcurlList);
@@ -513,7 +514,7 @@ int HTTP_OP::libcurl_exec(
 
 #if 0
         // HTTP PUT method
-        curl_off_t fs = (curl_off_t) content_length; 
+        curl_off_t fs = (curl_off_t) content_length;
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_POSTFIELDS, NULL);
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_INFILESIZE, content_length);
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_READDATA, fileIn);
@@ -524,16 +525,16 @@ int HTTP_OP::libcurl_exec(
         // HTTP POST method
         // set the multipart form for the file --
         // boinc just has the one section (file)
-        
+
 #if 0
-        //  if we ever want to do POST as multipart forms someday 
+        //  if we ever want to do POST as multipart forms someday
         // (many seem to prefer it that way, i.e. libcurl)
         //
         pcurlFormStart = pcurlFormEnd = NULL;
-        curl_formadd(&pcurlFormStart, &pcurlFormEnd, 
+        curl_formadd(&pcurlFormStart, &pcurlFormEnd,
                CURLFORM_FILECONTENT, infile,
                CURLFORM_CONTENTSLENGTH, content_length,
-               CURLFORM_CONTENTTYPE, g_content_type, 
+               CURLFORM_CONTENTTYPE, g_content_type,
                CURLFORM_END);
         curl_formadd(&post, &last,
                CURLFORM_COPYNAME, "logotype-image",
@@ -541,7 +542,7 @@ int HTTP_OP::libcurl_exec(
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_HTTPPOST, pcurlFormStart);
 #endif
 
-        curl_off_t fs = (curl_off_t) content_length; 
+        curl_off_t fs = (curl_off_t) content_length;
 
         pByte = NULL;
         lSeek = 0;    // initialize the vars we're going to use for byte transfers
@@ -556,7 +557,7 @@ int HTTP_OP::libcurl_exec(
         //
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_READDATA, this);
 
-        // callback function to rewind input file 
+        // callback function to rewind input file
         //
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_IOCTLFUNCTION, libcurl_ioctl);
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_IOCTLDATA, this);
@@ -583,7 +584,7 @@ int HTTP_OP::libcurl_exec(
         static int trace_count = 0;
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_DEBUGFUNCTION, libcurl_debugfunction);
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_DEBUGDATA, this );
-        curlErr = curl_easy_setopt(curlEasy, CURLOPT_VERBOSE, 1L); 
+        curlErr = curl_easy_setopt(curlEasy, CURLOPT_VERBOSE, 1L);
         trace_id = trace_count++;
     }
 
@@ -615,9 +616,8 @@ HTTP_OP_SET::HTTP_OP_SET() {
 
 // Adds an HTTP_OP to the set
 //
-int HTTP_OP_SET::insert(HTTP_OP* ho) {
+void HTTP_OP_SET::insert(HTTP_OP* ho) {
     http_ops.push_back(ho);
-    return 0;
 }
 
 // Remove an HTTP_OP from the set
@@ -638,27 +638,7 @@ int HTTP_OP_SET::remove(HTTP_OP* p) {
 }
 
 int HTTP_OP_SET::nops() {
-    return (int)http_ops.size();  
-}
-
-
-int HTTP_OP::set_proxy(PROXY_INFO *new_pi) {
-    pi.use_http_proxy = new_pi->use_http_proxy;
-    strcpy(pi.http_user_name, new_pi->http_user_name);
-    strcpy(pi.http_user_passwd, new_pi->http_user_passwd);
-    strcpy(pi.http_server_name, new_pi->http_server_name);
-    pi.http_server_port = new_pi->http_server_port;
-    pi.use_http_auth = new_pi->use_http_auth;
-	strcpy(pi.noproxy_hosts, new_pi->noproxy_hosts);
-
-    pi.use_socks_proxy = new_pi->use_socks_proxy;
-    strcpy(pi.socks5_user_name, new_pi->socks5_user_name);
-    strcpy(pi.socks5_user_passwd, new_pi->socks5_user_passwd);
-    strcpy(pi.socks_server_name, new_pi->socks_server_name);
-    pi.socks_server_port = new_pi->socks_server_port;
-    pi.socks_version = new_pi->socks_version;
-
-    return 0;
+    return (int)http_ops.size();
 }
 
 size_t libcurl_write(void *ptr, size_t size, size_t nmemb, HTTP_OP* phop) {
@@ -698,7 +678,7 @@ size_t libcurl_read( void *ptr, size_t size, size_t nmemb, HTTP_OP* phop) {
         // need to send headers first, then data file
         // so requests from 0 to strlen(req1)-1 are from memory,
         // and from strlen(req1) to content_length are from the file
-        if (phop->lSeek < (long) strlen(phop->req1)) {  
+        if (phop->lSeek < (long) strlen(phop->req1)) {
             // need to read header, either just starting to read
             // (i.e. this is the first time in this function for this phop)
             // or the last read didn't ask for the entire header
@@ -721,21 +701,21 @@ size_t libcurl_read( void *ptr, size_t size, size_t nmemb, HTTP_OP* phop) {
             // see if we're done with headers
             if (phop->lSeek >= (long) strlen(phop->req1)) {
                 phop->bSentHeader = true;
-                phop->lSeek = 0; 
+                phop->lSeek = 0;
             }
             return stRead;
         } else {
             // shouldn't happen
             phop->bSentHeader = true;
-            phop->lSeek = 0; 
+            phop->lSeek = 0;
         }
     }
-    if (phop->fileIn) { 
+    if (phop->fileIn) {
         long lFileSeek = phop->lSeek + (long) phop->file_offset;
         fseek(phop->fileIn, lFileSeek, SEEK_SET);
         if (!feof(phop->fileIn)) {
-            stRead = (int)fread(ptr, 1, stSend, phop->fileIn); 
-        }    
+            stRead = (int)fread(ptr, 1, stSend, phop->fileIn);
+        }
         phop->lSeek += (long) stRead;
         phop->bytes_xferred += (double)(stRead);
     }
@@ -767,7 +747,7 @@ int libcurl_debugfunction(
     char hdr[100];
     char buf[1024];
     size_t mysize;
-    
+
     switch (type) {
     case CURLINFO_TEXT:
         if (log_flags.http_debug) {
@@ -799,54 +779,45 @@ int libcurl_debugfunction(
 }
 
 
-void HTTP_OP::setupProxyCurl(bool no_proxy) {
-// PROXY_INFO pi useful members:
-//  pi.http_server_name
-//  pi.http_server_port
-//  pi.http_user_name
-//  pi.http_user_passwd
-//  pi.socks5_user_name
-//  pi.socks5_user_passwd
-//  pi.socks_server_name
-//  pi.socks_server_port
-//  pi.socks_version
-//  pi.use_http_auth
-//  pi.use_http_proxy
-//  pi.use_socks_proxy
-//
 // Curl self-explanatory setopt params for proxies:
 //    CURLOPT_HTTPPROXYTUNNEL
 //    CURLOPT_PROXYTYPE  (pass in CURLPROXY_HTTP or CURLPROXY_SOCKS5)
 //    CURLOPT_PROXYPORT  -- a long port #
 //    CURLOPT_PROXY - pass in char* of the proxy url
 //    CURLOPT_PROXYUSERPWD -- a char* in the format username:password
-//    CURLOPT_HTTPAUTH -- pass in one of CURLAUTH_BASIC, CURLAUTH_DIGEST, 
+//    CURLOPT_HTTPAUTH -- pass in one of CURLAUTH_BASIC, CURLAUTH_DIGEST,
 //        CURLAUTH_GSSNEGOTIATE, CURLAUTH_NTLM, CURLAUTH_ANY, CURLAUTH_ANYSAFE
 //    CURLOPT_PROXYAUTH -- "or" | the above bitmasks -- only basic, digest, ntlm work
-
+void HTTP_OP::setup_proxy_session(bool no_proxy) {
     CURLcode curlErr;
 
-    // CMC Note: the string szCurlProxyUserPwd must remain in memory
+    // CMC Note: the string m_curl_user_credentials must remain in memory
     // outside of this method (libcurl relies on it later when it makes
     // the proxy connection), so it has been placed as a member data for HTTP_OP
-    memset(szCurlProxyUserPwd,0x00,128);
+    //
+    strcpy(m_curl_user_credentials, "");
 
-	if (no_proxy) {
-		curlErr = curl_easy_setopt(curlEasy,CURLOPT_PROXY,"");
-		return;
-	}
+    if (no_proxy) {
+        curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXY, "");
+        return;
+    }
+
+    pi = gstate.proxy_info;
+    if (pi.use_http_proxy) {
+        if (log_flags.proxy_debug) {
+            msg_printf(
+                0, MSG_INFO, "[proxy_debug]: setting up proxy %s:%d",
+                pi.http_server_name, pi.http_server_port
+            );
+        }
 
-	if (pi.use_http_proxy) {
-		if (log_flags.proxy_debug) {
-			msg_printf(0, MSG_INFO,"[proxy_debug]: setting up proxy %s:%d",pi.http_server_name,pi.http_server_port);
-	    }
         // setup a basic http proxy
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYTYPE, CURLPROXY_HTTP);
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYPORT, (long) pi.http_server_port);
         curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXY, (char*) pi.http_server_name);
 
         if (pi.use_http_auth) {
-            if        (config.force_auth == "basic") {
+            if (config.force_auth == "basic") {
                 curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYAUTH, CURLAUTH_BASIC);
             } else if (config.force_auth == "digest") {
                 curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYAUTH, CURLAUTH_DIGEST);
@@ -857,30 +828,53 @@ void HTTP_OP::setupProxyCurl(bool no_proxy) {
             } else {
                 curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYAUTH, CURLAUTH_ANY);
             }
-            sprintf(szCurlProxyUserPwd, "%s:%s", pi.http_user_name, pi.http_user_passwd);
-            curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYUSERPWD, szCurlProxyUserPwd);
+            sprintf(m_curl_user_credentials, "%s:%s", pi.http_user_name, pi.http_user_passwd);
+            curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYUSERPWD, m_curl_user_credentials);
         }
-    } else {    
-        if (pi.use_socks_proxy) {
-            //pi.socks_version -- picks between socks5 & socks4 -- but libcurl only socks5!
-            curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS5);
-            curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYPORT, (long) pi.socks_server_port);
-            curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXY, (char*) pi.socks_server_name);
-            // libcurl uses blocking sockets with socks proxy, so limit timeout.
-            // - imlemented with local patch to libcurl
-            curlErr = curl_easy_setopt(curlEasy, CURLOPT_CONNECTTIMEOUT, 20L);
 
-            if (
-                strlen(pi.socks5_user_passwd)>0 || strlen(pi.socks5_user_name)>0
-            ) {
-                sprintf(szCurlProxyUserPwd, "%s:%s", pi.socks5_user_name, pi.socks5_user_passwd);
-                curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYUSERPWD, szCurlProxyUserPwd);
-                curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYAUTH, CURLAUTH_ANY & ~CURLAUTH_NTLM);
-            }
+    } else if (pi.use_socks_proxy) {
+
+        // pi.socks_version selects between socks5 & socks4.
+        // But libcurl only supports socks5, so ignore it.
+        //
+        curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS5);
+        curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYPORT, (long) pi.socks_server_port);
+        curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXY, (char*) pi.socks_server_name);
+        // libcurl uses blocking sockets with socks proxy, so limit timeout.
+        // - imlemented with local patch to libcurl
+        curlErr = curl_easy_setopt(curlEasy, CURLOPT_CONNECTTIMEOUT, 20L);
+
+        if (
+            strlen(pi.socks5_user_passwd) || strlen(pi.socks5_user_name)
+        ) {
+            sprintf(m_curl_user_credentials, "%s:%s", pi.socks5_user_name, pi.socks5_user_passwd);
+            curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYUSERPWD, m_curl_user_credentials);
+            curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYAUTH, CURLAUTH_ANY & ~CURLAUTH_NTLM);
         }
+    } else if (pi.have_autodetect_proxy_settings && strlen(pi.autodetect_server_name)) {
+        if (log_flags.proxy_debug) {
+            msg_printf(0, MSG_INFO,
+                "[proxy_debug] HTTP_OP::setup_proxy_session(): setting up automatic proxy %s:%d",
+                pi.autodetect_server_name, pi.autodetect_port
+            );
+        }
+
+        switch(pi.autodetect_protocol) {
+        case URL_PROTOCOL_SOCKS:
+            curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS5);
+            break;
+        case URL_PROTOCOL_HTTP:
+        case URL_PROTOCOL_HTTPS:
+        default:
+            curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYTYPE, CURLPROXY_HTTP);
+            break;
+        }
+        curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXYPORT, (long) pi.autodetect_port);
+        curlErr = curl_easy_setopt(curlEasy, CURLOPT_PROXY, (char*) pi.autodetect_server_name);
     }
 }
 
+
 // the file descriptor sets need to be global so libcurl has access always
 //
 fd_set read_fds, write_fds, error_fds;
@@ -948,14 +942,14 @@ void HTTP_OP::handle_messages(CURLMsg *pcurlMsg) {
     CURLcode curlErr;
     int retval;
 
-    curlErr = curl_easy_getinfo(curlEasy, 
+    curlErr = curl_easy_getinfo(curlEasy,
         CURLINFO_RESPONSE_CODE, &response
     );
 
     // CURLINFO_LONG+25 is a workaround for a bug in the gcc version
     // included with Mac OS X 10.3.9
     //
-    curlErr = curl_easy_getinfo(curlEasy, 
+    curlErr = curl_easy_getinfo(curlEasy,
         (CURLINFO)(CURLINFO_LONG+25) /*CURLINFO_OS_ERRNO*/, &connect_error
     );
 
@@ -971,13 +965,13 @@ void HTTP_OP::handle_messages(CURLMsg *pcurlMsg) {
         // (we don't use it)
         //
         double size_download, total_time, starttransfer_time;
-        curlErr = curl_easy_getinfo(curlEasy, 
+        curlErr = curl_easy_getinfo(curlEasy,
             CURLINFO_SIZE_DOWNLOAD, &size_download
         );
-        curlErr = curl_easy_getinfo(curlEasy, 
+        curlErr = curl_easy_getinfo(curlEasy,
             CURLINFO_TOTAL_TIME, &total_time
         );
-        curlErr = curl_easy_getinfo(curlEasy, 
+        curlErr = curl_easy_getinfo(curlEasy,
             CURLINFO_STARTTRANSFER_TIME, &starttransfer_time
         );
         double dt = total_time - starttransfer_time;
@@ -987,13 +981,13 @@ void HTTP_OP::handle_messages(CURLMsg *pcurlMsg) {
     }
     if (want_upload) {
         double size_upload, total_time, starttransfer_time;
-        curlErr = curl_easy_getinfo(curlEasy, 
+        curlErr = curl_easy_getinfo(curlEasy,
             CURLINFO_SIZE_UPLOAD, &size_upload
         );
-        curlErr = curl_easy_getinfo(curlEasy, 
+        curlErr = curl_easy_getinfo(curlEasy,
             CURLINFO_TOTAL_TIME, &total_time
         );
-        curlErr = curl_easy_getinfo(curlEasy, 
+        curlErr = curl_easy_getinfo(curlEasy,
             CURLINFO_STARTTRANSFER_TIME, &starttransfer_time
         );
         double dt = total_time - starttransfer_time;
@@ -1004,12 +998,12 @@ void HTTP_OP::handle_messages(CURLMsg *pcurlMsg) {
 
     // the op is done if curl_multi_msg_read gave us a msg for this http_op
     //
-    http_op_state = HTTP_STATE_DONE;        
+    http_op_state = HTTP_STATE_DONE;
     CurlResult = pcurlMsg->data.result;
 
     if (CurlResult == CURLE_OK) {
         if ((response/100)*100 == HTTP_STATUS_OK) {
-            http_op_retval = 0;  
+            http_op_retval = 0;
         } else if ((response/100)*100 == HTTP_STATUS_CONTINUE) {
             return;
         } else {
@@ -1034,6 +1028,7 @@ void HTTP_OP::handle_messages(CURLMsg *pcurlMsg) {
         strcpy(error_msg, curl_easy_strerror(CurlResult));
         switch(CurlResult) {
         case CURLE_COULDNT_RESOLVE_HOST:
+            reset_dns();
             http_op_retval = ERR_GETHOSTBYNAME;
             break;
         case CURLE_COULDNT_CONNECT:
@@ -1068,7 +1063,7 @@ void HTTP_OP::handle_messages(CURLMsg *pcurlMsg) {
             if (dSize >= (size_t)req1_len) {
                 dSize = req1_len-1;
             }
-            size_t nread = fread(req1, 1, dSize, fileOut); 
+            size_t nread = fread(req1, 1, dSize, fileOut);
             if (nread != dSize) {
                 if (log_flags.http_debug) {
                     msg_printf(NULL, MSG_INFO,
@@ -1112,9 +1107,9 @@ void HTTP_OP_SET::got_select(FDSET_GROUP&, double timeout) {
 
     // read messages from curl that may have come in from the above loop
     //
-	while (1) {
-		pcurlMsg = curl_multi_info_read(g_curlMulti, &iNumMsg);
-		if (!pcurlMsg) break;
+    while (1) {
+        pcurlMsg = curl_multi_info_read(g_curlMulti, &iNumMsg);
+        if (!pcurlMsg) break;
 
         // if we have a msg, then somebody finished
         // can check also with pcurlMsg->msg == CURLMSG_DONE
@@ -1128,8 +1123,8 @@ void HTTP_OP_SET::got_select(FDSET_GROUP&, double timeout) {
 // Return the HTTP_OP object with given Curl object
 //
 HTTP_OP* HTTP_OP_SET::lookup_curl(CURL* pcurl)  {
-	for (unsigned int i=0; i<http_ops.size(); i++) {
-		if (http_ops[i]->curlEasy == pcurl) {
+    for (unsigned int i=0; i<http_ops.size(); i++) {
+        if (http_ops[i]->curlEasy == pcurl) {
             return http_ops[i];
         }
     }
@@ -1148,7 +1143,7 @@ void HTTP_OP::update_speed() {
 
 void HTTP_OP::set_speed_limit(bool is_upload, double bytes_sec) {
 #if LIBCURL_VERSION_NUM >= 0x070f05
-	CURLcode cc = CURLE_OK;
+    CURLcode cc = CURLE_OK;
     curl_off_t bs = (curl_off_t)bytes_sec;
 
     if (is_upload) {
@@ -1156,9 +1151,12 @@ void HTTP_OP::set_speed_limit(bool is_upload, double bytes_sec) {
     } else {
         cc = curl_easy_setopt(curlEasy, CURLOPT_MAX_RECV_SPEED_LARGE, bs);
     }
-	if (cc && log_flags.http_debug) {
-		msg_printf(NULL, MSG_INFO, "[http_debug] Curl error in set_speed_limit(): %s", curl_easy_strerror(cc));
-	}
+    if (cc && log_flags.http_debug) {
+        msg_printf(NULL, MSG_INFO,
+            "[http_debug] Curl error in set_speed_limit(): %s",
+            curl_easy_strerror(cc)
+        );
+    }
 #endif
 }
 
@@ -1176,4 +1174,4 @@ void HTTP_OP_SET::cleanup_temp_files() {
     dir_close(d);
 }
 
-const char *BOINC_RCSID_57f273bb60 = "$Id: http_curl.cpp 16189 2008-10-11 18:13:33Z davea $";
+const char *BOINC_RCSID_57f273bb60 = "$Id: http_curl.cpp 19101 2009-09-18 20:48:19Z romw $";
